/******************************************************************************
 app_flash_memory.c - flash memory source file 
 Version: 0.9.1
 Created on: 03/07/2021
 Created by: Rohit Rajapure

 Copyright (c) 2017 Shalaka Connected Devices LLP.  All rights reserved.
 Software License Agreement

   Redistribution and use in source and binary forms, with or without
   modification, are not permitted.

   Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

   Neither the name of Shalaka Connected Devices LLP nor the names of
   its contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "string.h"
#include "sdk_config.h"
#include "zboss_api.h"
#include "zboss_api_addons.h"

#include "bsp.h"
#include "boards.h"
#include "nrf_delay.h"
#include "app_error.h"

//flash include files 
#include "nrf_fstorage.h"
#include "nrf_fstorage_nvmc.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "app_flash_memory.h"

/*****************************************************************************************
* GLOBAL VARIABLES
*
******************************************************************************************/
uint32_t ui32SaveDataFlash = 0;

/******************************************************************************************
*EXTERNAL VARIABLES
*
*******************************************************************************************/
extern nrf_fstorage_t fstorage; 
/******************************************************************************************
*FUNCTIONS 
*
*******************************************************************************************/
/*****************Flash functions and callback start*************************************/

//Flash related callback function initialization
static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt);

//structure defines the callback function, flash start and end address 
NRF_FSTORAGE_DEF(nrf_fstorage_t fstorage) =
{
    /* Set a handler for fstorage events. */
    .evt_handler = fstorage_evt_handler,

    /* These below are the boundaries of the flash space assigned to this instance of fstorage.
     * You must set these manually, even at runtime, before nrf_fstorage_init() is called.
     * The function nrf5_flash_end_addr_get() can be used to retrieve the last address on the
     * last page of flash available to write data. */
    .start_addr = FLASH_START_ADDRESS,
    .end_addr   = FLASH_END_ADDRESS,
};
//fstorage event handler/callback function 
static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt)
{
    if (p_evt->result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("--> Event received: ERROR while executing an fstorage operation.");
        return;
    }

    switch (p_evt->id)
    {
        case NRF_FSTORAGE_EVT_WRITE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: wrote %d bytes at address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        case NRF_FSTORAGE_EVT_ERASE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: erased %d page from address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        default:
            break;
    }
}

/****************************************************************************
 *
 * NAME: Flashinitialization
 *
 * DESCRIPTION:
 * function for initialization flash memory
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void Flashinitialization(void)
{
    ret_code_t rc;
    nrf_fstorage_api_t * p_fs_api;

  #ifdef SOFTDEVICE_PRESENT
      NRF_LOG_INFO("SoftDevice is present.");
      NRF_LOG_INFO("Initializing nrf_fstorage_sd implementation...");
      /* Initialize an fstorage instance using the nrf_fstorage_sd backend.
       * nrf_fstorage_sd uses the SoftDevice to write to flash. This implementation can safely be
       * used whenever there is a SoftDevice, regardless of its status (enabled/disabled). */
      p_fs_api = &nrf_fstorage_sd;
  #else
      NRF_LOG_INFO("SoftDevice not present.");
      NRF_LOG_INFO("Initializing nrf_fstorage_nvmc implementation...");
      /* Initialize an fstorage instance using the nrf_fstorage_nvmc backend.
       * nrf_fstorage_nvmc uses the NVMC peripheral. This implementation can be used when the
       * SoftDevice is disabled or not present.
       *
       * Using this implementation when the SoftDevice is enabled results in a hardfault. */
      p_fs_api = &nrf_fstorage_nvmc;
  #endif

    rc = nrf_fstorage_init(&fstorage, p_fs_api, NULL);
    APP_ERROR_CHECK(rc);
}

/*****************Flash functions and callback end**************************************/
/****************************************************************************
 *
 * NAME: WriteFlashMemory
 *
 * DESCRIPTION:
 * function for write data to flash memory
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void WriteFlashMemory (void)
{
    ret_code_t     rc;
    rc = nrf_fstorage_write(&fstorage,FLASH_START_ADDRESS, &ui32SaveDataFlash, sizeof(ui32SaveDataFlash), NULL);    //write data from flash
    APP_ERROR_CHECK(rc);
}

/****************************************************************************
 *
 * NAME: ReadFlashMemory
 *
 * DESCRIPTION:
 * function for read data from flash memory
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void ReadFlashMemory (void)
{
    ret_code_t     rc;
    rc = nrf_fstorage_read(&fstorage,FLASH_START_ADDRESS,&ui32SaveDataFlash,sizeof(ui32SaveDataFlash));   //read data from flash
    APP_ERROR_CHECK(rc);
}

/****************************************************************************
 *
 * NAME: EraseFlashMemory
 *
 * DESCRIPTION:
 * function for erasing flash memory
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void EraseFlashMemory (void)
{
  ret_code_t     rc;
  rc = nrf_fstorage_erase(&fstorage,FLASH_START_ADDRESS,1, NULL);   //erase data from flash
  APP_ERROR_CHECK(rc);
}


 

 